require 'net/http'
require 'open-uri'
require 'yaml'
require 'jenkins_api_client'
require 'date'

require "autodeploy/version"
require 'active_record'
require File.expand_path('../autodeploy/models/deploy_build', __FILE__)
require File.expand_path('../autodeploy/models/deploy_lock', __FILE__)
require File.expand_path('../autodeploy/models/deploy_audit', __FILE__)
require File.expand_path('../autodeploy/deployer', __FILE__)



module Autodeploy
  def self.log(message)
    puts "[#{DateTime.now.strftime}] #{message}"
  end

  class Job

    def initialize(configfile)
      @config = YAML.load_file(configfile)
      @client = JenkinsApi::Client.new(@config)

      unless Dir.exist? @config['download_dir']
        Autodeploy.log "Creating #{@config['download_dir']} in #{Dir.pwd}"
        Dir.mkdir @config['download_dir']
      end

      ActiveRecord::Base.establish_connection(@config['database'])
    end

    def job_number
      @job_number ||= @client.job.get_current_build_number(@config['job'])
    end

    def job
      @job ||= @client.job.get_build_details(@config['job'], job_number)
    end

    def run
      @deployer = Autodeploy::Deployer.new(@config, job)
      @deployer.run()

      return unless job['result'] == 'SUCCESS'

      build_action = job['actions'].detect{|action| action.include?('lastBuiltRevision')}
      sha = build_action.nil? ? nil : build_action['lastBuiltRevision']['SHA1']

      filename = [job_number, sha, job['artifacts'][0]['fileName']].compact.join('-')
      filepath = File.join(@config['download_dir'], filename)

      if File.exists?(filepath)
        Autodeploy.log "#{filepath} already exists"
        return
      end

      uri = URI("#{job['url']}artifact/#{job['artifacts'][0]['relativePath']}")

      Autodeploy.log "Starting download: #{uri.path}"

      Net::HTTP.start(uri.host, uri.port, use_ssl: true, verify_mode: OpenSSL::SSL::VERIFY_NONE) do |http|
        request = Net::HTTP::Get.new uri.request_uri
        request.basic_auth @config['username'], @config['password']

        resp = http.request(request)

        if resp.code != "200"
          Autodeploy.log "Error downloading artifact: #{resp.code} - #{resp.message}"
          return
        end

        Autodeploy.log "Saving #{filename}"

        open("downloads/#{filename}", "wb") do |file|
          file.write(resp.body)
          Autodeploy.log "Completed download - #{filename}"
        end
      end
    end

    def daemonize(time=10)
      loop do
        run
        sleep time
      end
    end
  end
end
