require 'socket'

module Autodeploy
	class Deployer
		def initialize(yaml_config, jenkins_job)
			@config = yaml_config
			@job = jenkins_job
		end

		def run
      return unless ready_to_deploy?(@config['job'])

			deploy_build = get_build_to_deploy(@config['job'])
			build_full_file_path(deploy_build)

			file_downloaded = has_file_been_downloaded
			return unless file_downloaded

			file_deployed = has_file_been_deployed
			return if file_deployed

      lock_and_deploy
		end

    def lock_and_deploy
      locked = acquire_lock_from_mysql
      if locked
        deploy
      else
        Autodeploy.log "Waiting for lock..."
      end
    end

    def deploy
      Autodeploy.log "Lock acquired"

      begin
        return unless move_artifact_to_deploy_dir

        record_successful_deployment
      ensure
        release_lock_from_mysql
      end
    end

    def build_full_file_path(deploy_build)
			artifact_file_name = @job['artifacts'][0]['fileName']

			Autodeploy.log "DeployBuild: #{deploy_build}"

			filename = [deploy_build.build_number, deploy_build.git_sha, artifact_file_name].compact.join('-')
			@filepath = File.join(@config['download_dir'], filename)
    end

    def ready_to_deploy?(job_name)
      Models::DeployBuild.where(job_name: job_name).order("created_at DESC").exists?
    end

		def get_build_to_deploy(job_name)
        Models::DeployBuild.where(job_name: job_name).order("created_at DESC").first
		end

		def has_file_been_downloaded
			File.exists?(@filepath)
    end

    def record_successful_deployment
      Models::DeployAudit.create(host: Socket.gethostname,
                                 build_number: @job['number'],
                                 jenkins_build_timestamp: @job['timestamp'],
                                 created_at:  Time.now.to_i * 1000)
    end

		def has_file_been_deployed
      Models::DeployAudit.where(host: Socket.gethostname,
                                build_number: @job['number'],
                                jenkins_build_timestamp: @job['timestamp']).exists?
		end

		def acquire_lock_from_mysql
      Autodeploy.log "Attempting to acquire lock..."

			begin
				Models::DeployLock.create(job_name: @config['job'],
                                  host: Socket.gethostname,
                                  created_at: Time.now.to_i * 1000)
			rescue ActiveRecord::RecordNotUnique
				nil
			end
		end

		def release_lock_from_mysql
      Autodeploy.log "Releasing lock..."
			lock = Models::DeployLock.where(job_name: @config['job'], host: Socket.gethostname).first
			lock.delete
		end

		def move_artifact_to_deploy_dir
      Autodeploy.log "Moving aritifact to deploy dir"

			cmd = "mv -f #{@filepath} #{@config['deploy_dir']}/#{@config['deploy_artifact_name']}"
			output = system(cmd)

      Autodeploy.log output

      output
		end
	end
end